from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.add_workspace_users_request import AddWorkspaceUsersRequest
from ...models.http_validation_error import HTTPValidationError
from ...models.workspace_user_response import WorkspaceUserResponse
from typing import cast



def _get_kwargs(
    *,
    body: AddWorkspaceUsersRequest,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/workspace/users",
    }

    _kwargs["json"] = body.to_dict()


    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> HTTPValidationError | list[WorkspaceUserResponse] | None:
    if response.status_code == 201:
        response_201 = []
        _response_201 = response.json()
        for response_201_item_data in (_response_201):
            response_201_item = WorkspaceUserResponse.from_dict(response_201_item_data)



            response_201.append(response_201_item)

        return response_201

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())



        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[HTTPValidationError | list[WorkspaceUserResponse]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: AddWorkspaceUsersRequest,

) -> Response[HTTPValidationError | list[WorkspaceUserResponse]]:
    """ Add Workspace Users

     Add users to a workspace (bulk operation). Only workspace owners can add users.

    Client provides workspace-scoped JWT. Server decrypts workspace key from JWT claims,
    then wraps it with each recipient's public key.
    Returns the full WorkspaceUserResponse for each successfully added user.

    Args:
        body (AddWorkspaceUsersRequest): Request to add users to a workspace (POST
            /workspace/{id}/users).

            All invited users receive the same role.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | list[WorkspaceUserResponse]]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: AuthenticatedClient | Client,
    body: AddWorkspaceUsersRequest,

) -> HTTPValidationError | list[WorkspaceUserResponse] | None:
    """ Add Workspace Users

     Add users to a workspace (bulk operation). Only workspace owners can add users.

    Client provides workspace-scoped JWT. Server decrypts workspace key from JWT claims,
    then wraps it with each recipient's public key.
    Returns the full WorkspaceUserResponse for each successfully added user.

    Args:
        body (AddWorkspaceUsersRequest): Request to add users to a workspace (POST
            /workspace/{id}/users).

            All invited users receive the same role.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | list[WorkspaceUserResponse]
     """


    return sync_detailed(
        client=client,
body=body,

    ).parsed

async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: AddWorkspaceUsersRequest,

) -> Response[HTTPValidationError | list[WorkspaceUserResponse]]:
    """ Add Workspace Users

     Add users to a workspace (bulk operation). Only workspace owners can add users.

    Client provides workspace-scoped JWT. Server decrypts workspace key from JWT claims,
    then wraps it with each recipient's public key.
    Returns the full WorkspaceUserResponse for each successfully added user.

    Args:
        body (AddWorkspaceUsersRequest): Request to add users to a workspace (POST
            /workspace/{id}/users).

            All invited users receive the same role.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | list[WorkspaceUserResponse]]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    body: AddWorkspaceUsersRequest,

) -> HTTPValidationError | list[WorkspaceUserResponse] | None:
    """ Add Workspace Users

     Add users to a workspace (bulk operation). Only workspace owners can add users.

    Client provides workspace-scoped JWT. Server decrypts workspace key from JWT claims,
    then wraps it with each recipient's public key.
    Returns the full WorkspaceUserResponse for each successfully added user.

    Args:
        body (AddWorkspaceUsersRequest): Request to add users to a workspace (POST
            /workspace/{id}/users).

            All invited users receive the same role.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | list[WorkspaceUserResponse]
     """


    return (await asyncio_detailed(
        client=client,
body=body,

    )).parsed
